import numpy as np
from sklearn.preprocessing import LabelBinarizer

from imblearn.under_sampling import RandomUnderSampler

from spkm.spkm_wrapper import SPKMl2l1likeSklearn
from spkm.kernels_and_gradients import RBFKernel, PolyKernel

from ..monoview.monoview_utils import BaseMonoviewClassifier
from ..utils.hyper_parameter_search import CustomRandint
from ..monoview_classifiers.spkm import SPKM

classifier_class_name = "BaggedSPKM"
proto=True

class BaggedSPKM(BaseMonoviewClassifier, SPKMl2l1likeSklearn):

    def __init__(self, random_state=42, n_u=2, kernel=RBFKernel(0.5),
                 spkmregP_01=1, spkminit="randn",
                 nspkminits=10, preprocessinglist=[0,1,2],
                 spkm_l1_regstyle="l1ball", **kwargs):

        SPKM.__init__(self, random_state=random_state,
                                         n_u=n_u,
                                         kernel=kernel,
                                         spkmregP_01=spkmregP_01,
                                         spkminit=spkminit,
                                         nspkminits=nspkminits,
                                         preprocessinglist=preprocessinglist,
                                         spkm_l1_regstyle=spkm_l1_regstyle)
        self.rus = RandomUnderSampler(random_state=random_state)

    def fit(self, X, y):
        self.lb =  LabelBinarizer(pos_label=1, neg_label=-1)
        y = self.lb.fit_transform(y)
        return SPKMl2l1likeSklearn.fit(self, X, y[:,0],)

    def predict(self, X, preprocess=True):
        return self.lb.inverse_transform(np.sign(SPKMl2l1likeSklearn.predict(self, X)))

    def get_interpretation(self, directory, base_file_name, labels, multiclass=False):
        u = self.feature_interpretability()
        importances_sum = np.sum(u)
        self.feature_importances_ = u/importances_sum
        return ""

    def accepts_multi_class(self, random_state, n_samples=10, dim=2,
                            n_classes=3, n_views=2):
        return False