import numpy as np
from sklearn.linear_model import Lasso as LassoSK
from sklearn.preprocessing import RobustScaler

from ..monoview.monoview_utils import BaseMonoviewClassifier
from summit.multiview_platform.utils.hyper_parameter_search import CustomUniform, CustomRandint

# Author-Info
__author__ = "Baptiste Bauvin"
__status__ = "Prototype"  # Production, Development, Prototype

classifier_class_name = "Lasso"


class Lasso(LassoSK, BaseMonoviewClassifier):
    """
     This class is an adaptation of scikit-learn's `Lasso <https://scikit-learn.org/stable/modules/generated/sklearn.linear_model.Lasso.html>`_


     """

    def __init__(self, random_state=None, alpha=1.0,
                 max_iter=10, warm_start=False, scale=False, **kwargs):
        LassoSK.__init__(self,
                         alpha=alpha,
                         max_iter=max_iter,
                         warm_start=warm_start,
                         random_state=random_state
                         )
        self.scale = scale
        self.param_names = ["max_iter", "alpha", "scale", "random_state"]
        self.classed_params = []
        self.distribs = [CustomRandint(low=1, high=300),
                         CustomUniform(), [True, False], [random_state]]
        self.weird_strings = {}

    def fit(self, X, y, check_input=True):
        neg_y = np.copy(y)
        neg_y[np.where(neg_y == 0)] = -1
        if self.scale:
            self.scaler = RobustScaler()
            X = self.scaler.fit_transform(X, neg_y)
        LassoSK.fit(self, X, neg_y)
        if np.sum(self.coef_)!=0:
            self.feature_importances_ = np.abs(self.coef_/np.sum(self.coef_))
        else:
            self.feature_importances_ = np.abs(self.coef_)
        return self

    def predict(self, X):
        if self.scale:
            X = self.scaler.transform(X)
        prediction = LassoSK.predict(self, X)
        signed = np.sign(prediction)
        signed[np.where(signed == -1)] = 0
        return signed

    def get_interpretation(self, directory, base_file_name, y_test, feature_ids,
                           multi_class=False):  # pragma: no cover
        interpretString = ""
        interpretString += self.get_feature_importance(directory,
                                                       base_file_name,
                                                       feature_ids)
        # interpretString += "\n\n Estimator error | Estimator weight\n"
        # interpretString += "\n".join(
        #     [str(error) + " | " + str(weight / sum(self.estimator_weights_)) for
        #      error, weight in
        #      zip(self.estimator_errors_, self.estimator_weights_)])
        # step_test_metrics = np.array(
        #     [self.plotted_metric.score(y_test, step_pred) for step_pred in
        #      self.step_predictions])
        # get_accuracy_graph(step_test_metrics, "Adaboost",
        #                    os.path.join(directory,
        #                                 base_file_name + "test_metrics.png"),
        #                    self.plotted_metric_name, set="test")
        # np.savetxt(os.path.join(directory, base_file_name + "test_metrics.csv"),
        #            step_test_metrics,
        #            delimiter=',')
        # np.savetxt(
        #     os.path.join(directory, base_file_name + "train_metrics.csv"),
        #     self.metrics, delimiter=',')
        # np.savetxt(os.path.join(directory, base_file_name + "times.csv"),
        #            np.array([self.train_time, self.pred_time]), delimiter=',')
        return interpretString