import os
import time

import numpy as np
from sklearn.ensemble import AdaBoostClassifier
from sklearn.linear_model import RidgeClassifier, LogisticRegression
from lineartree import LinearTreeClassifier


from ..monoview.monoview_utils import BaseMonoviewClassifier
from summit.multiview_platform.utils.hyper_parameter_search import CustomRandint

# Author-Info
__author__ = "Baptiste Bauvin"
__status__ = "Prototype"  # Production, Development, Prototype

classifier_class_name = "LinearLeafsTree"

class LinearLeafsTree(LinearTreeClassifier, BaseMonoviewClassifier):
    """
    This class is an adaptation of scikit-learn's `AdaBoostClassifier <https://scikit-learn.org/stable/modules/generated/sklearn.ensemble.AdaBoostClassifier.html#sklearn.ensemble.AdaBoostClassifier>`_

    """

    def __init__(self, base_estimator=None,
                 base_estimator_config=None, max_depth=None, **kwargs):
        base_estimator = BaseMonoviewClassifier.get_base_estimator(self,
                                                                   base_estimator,
                                                                   base_estimator_config)
        LinearTreeClassifier.__init__(self,
                                    base_estimator=base_estimator,
                                      max_depth=max_depth
                                    )
        self.param_names = [ "base_estimator", "max_depth"]
        self.classed_params = ["base_estimator"]
        self.distribs = [[LogisticRegression(), RidgeClassifier()], CustomRandint(low=1, high=20)]
        self.weird_strings = {"base_estimator": "class_name"}
        self.base_estimator_config = base_estimator_config

    def get_interpretation(self, directory, base_file_name, y_test, feature_ids,
                           multi_class=False):  # pragma: no cover
        interpretString = ""
        return interpretString
