import numpy as np

from ..monoview.monoview_utils import BaseMonoviewClassifier
from summit.multiview_platform.utils.hyper_parameter_search import CustomUniform

classifier_class_name = "LWLR"
class LWLR(BaseMonoviewClassifier):

    def __init__(self, tau=0.05, reg=0.0001, threshold=1e-6, random_state=42):
        self.reg = reg
        self.threshold = threshold
        self.tau = tau
        self.random_state = random_state
        self.param_names = ["tau", 'reg', "threshold"]
        self.distribs = [CustomUniform(loc=1e-2, state=1),
                         CustomUniform(loc=1e-6, state=1e-2),
                         CustomUniform(loc=1e-8, state=1e-4)]
        self.weird_strings={}
        self.classed_params=[]

    def weights(self, x_train, x):
        sq_diff = (x_train - x) ** 2
        norm_sq = sq_diff.sum(axis=1)
        return np.ravel(np.exp(- norm_sq / (2 * self.tau ** 2)))

    def logistic(self, x_train):
        return np.ravel(1 / (1 + np.exp(-x_train.dot(self.theta))))

    def fit(self, X, y, **fit_params):
        self.X = X
        self.y = y

    def train(self, x):
        self.w = self.weights(self.X, x)
        self.theta = np.zeros(self.X.shape[1])
        gradient = np.ones(self.X.shape[1]) * np.inf
        while np.linalg.norm(gradient) > self.threshold:
            # compute gradient
            h = self.logistic(self.X)
            gradient = self.X.T.dot(
                self.w * (np.ravel(self.y) - h)) - self.reg * self.theta
            # Compute Hessian
            D = np.diag(-(self.w * h * (1 - h)))
            H = self.X.T.dot(D).dot(self.X) - self.reg * np.identity(
                self.X.shape[1])
            # weight update
            self.theta = self.theta - np.linalg.inv(H).dot(gradient)

    def predict(self, X):
        preds = []
        for x in X:
            self.train(x)
            preds.append(np.array(self.logistic(X) > 0.5).astype(int)[0])
        return np.array(preds)
