import os
import time

import numpy as np
from xgboost import XGBClassifier
from sklearn.tree import DecisionTreeClassifier

from .. import metrics
from ..monoview.monoview_utils import BaseMonoviewClassifier, get_accuracy_graph
from summit.multiview_platform.utils.hyper_parameter_search import CustomRandint, CustomUniform

# Author-Info
__author__ = "Baptiste Bauvin"
__status__ = "Prototype"  # Production, Development, Prototype

classifier_class_name = "XGB"



class XGB(XGBClassifier, BaseMonoviewClassifier):
    """
     This class is an adaptation of scikit-learn's `GradientBoostingClassifier <https://scikit-learn.org/stable/modules/generated/sklearn.ensemble.GradientBoostingClassifier.html>`_


     """

    def __init__(self, random_state=None, learning_rate=0.3, max_depth=1.0,
                 n_estimators=100, objective='binary:logistic',
                 **kwargs):
        XGBClassifier.__init__(self, learning_rate=learning_rate, max_depth=max_depth,
                               n_estimators=n_estimators, objective=objective,
                               random_state=random_state)
        self.param_names = ["n_estimators", "learning_rate", "max_depth", "objective"]
        self.classed_params = []
        self.distribs = [CustomRandint(low=1, high=300),
                         CustomUniform(),
                         CustomRandint(low=1, high=10),
                         ['binary:logistic', 'binary:hinge', ],]
        self.weird_strings = {}


    def get_interpretation(self, directory, base_file_name, y_test, feature_ids,
                           multi_class=False):
        interpretString = ""
        if multi_class:
            return interpretString
        else:
            interpretString += self.get_feature_importance(directory,
                                                           base_file_name,
                                                           feature_ids)
            # step_test_metrics = np.array(
            #     [self.plotted_metric.score(y_test, step_pred) for step_pred in
            #      self.step_predictions])
            # get_accuracy_graph(step_test_metrics, "AdaboostClassic",
            #                    directory + "test_metrics.png",
            #                    self.plotted_metric_name, set="test")
            # get_accuracy_graph(self.metrics, "AdaboostClassic",
            #                    directory + "metrics.png",
            #                    self.plotted_metric_name)
            # np.savetxt(
            #     os.path.join(directory, base_file_name + "test_metrics.csv"),
            #     step_test_metrics,
            #     delimiter=',')
            # np.savetxt(
            #     os.path.join(directory, base_file_name + "train_metrics.csv"),
            #     self.metrics,
            #     delimiter=',')
            # np.savetxt(os.path.join(directory, base_file_name + "times.csv"),
            #            np.array([self.train_time, self.pred_time]),
            #            delimiter=',')
            return interpretString
